/*
StoreService on service, joka sisältää itse tehdyn reaktiivisen storen, 
joka soveltuu yksinkertaiseen mutta tehokkaaseen tilanhallintaan.
Signaaleja käytetään siksi, että tilanhallinta saadaan niiden avulla 
toimimaan reaktiivisesti.

Tilaa säilytetään signaalimuuttujissa:

orders (taulukko jossa tilausnumerot) 
saldo (käyttäjän tilin saldo) 
name (käyttäjän nimi) 

Aina kun signaalin arvo muuttuu StoreServicessä, lähtee ilmoitus muutoksesta
komponenttiin, joka vastaanottaa muutoksen. Komponentit laukaisevat storen metodeita, 
jotka päivittävät storen tilaa eli muuttavat signaalien arvoja StoreServicessä.
*/

import { Injectable, signal } from '@angular/core';

@Injectable({
  providedIn: 'root',
})
export class StoreService {
  
  /*************Storen muuttujat jotka säilyttävät tilaa**********/

  orders = signal<number[]>([]);

  saldo = signal<number>(0);

  name = signal<string>('Käyttäjä'); 

  /*************Storen metodit, jotka päivittävät tilaa***********/

  // lisätään tilausnumeroita
  public addOrder(id: number) {
    // entisten arvojen perään tulee uusi id
    const orders = [...this.orders(), id];
    // uusi tilaus orders-signaaliin
    this.orders.set(orders);
  }
  // päivitetään (pienennetään) saldoa, eli tehdään maksu
  public updateSaldo(payment: number) {
    if (this.saldo()) {
      const updatedSaldo = this.saldo() - payment;
      // päivitetty saldo balance-signaaliin
      this.saldo.set(updatedSaldo);
    }
  }
  // tallennetaan uusi saldo
  public saveSaldo(saldo: number) {
    // uusi saldo saldo-signaaliin
    this.saldo.set(saldo);
  }
  // tallennetaan uusi nimi
  public saveName(name: string) {
    // uusi nimi name-signaaliin
    this.name.set(name);
  }
}
